@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -102,16 +102,7 @@ class output ~kind ~clock_safe ~start ~on_start ~on_stop ~infallible buflen
       let buf = AFrame.pcm memo in
       self#handle "write_stream" (fun () ->
           let len = Audio.length buf in
-          (* TODO: non-interleaved format does not seem to be supported here *)
-          (*
-           let ba = Bigarray.Array2.create Bigarray.float32 Bigarray.c_layout channels len in
-           for c = 0 to channels - 1 do
-             Bigarray.Array1.blit buf.(c) (Bigarray.Array2.slice_left ba c)
-           done;
-           let ba = Bigarray.genarray_of_array2 ba in
-           *)
-          let ba = Bigarray.genarray_of_array1 (Audio.interleave buf) in
-          Portaudio.write_stream_ba stream ba 0 len)
+          Portaudio.write_stream stream buf 0 len)
   end
 
 class input ~kind ~clock_safe ~start ~on_start ~on_stop ~fallible buflen =
@@ -149,21 +140,7 @@ class input ~kind ~clock_safe ~start ~on_start ~on_stop ~fallible buflen =
       let stream = Option.get stream in
       let buf = AFrame.pcm frame in
       self#handle "read_stream" (fun () ->
-          let len = Audio.length buf in
-          let ibuf =
-            Bigarray.Array1.create Bigarray.float32 Bigarray.c_layout
-              (self#audio_channels * len)
-          in
-          Portaudio.read_stream_ba stream
-            (Bigarray.genarray_of_array1 ibuf)
-            0 len;
-          for c = 0 to self#audio_channels - 1 do
-            let bufc = buf.(c) in
-            for i = 0 to len - 1 do
-              bufc.{i} <-
-                Bigarray.Array1.unsafe_get ibuf ((i * self#audio_channels) + c)
-            done
-          done);
+          Portaudio.read_stream stream buf 0 (Array.length buf.(0)));
       AFrame.add_break frame (AFrame.size ())
   end
 
