@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -454,7 +454,17 @@ let conf_log =
   Dtools.Conf.bool ~p:(conf_srt#plug "log") ~d:true
     "Route srt logs through liquidsoap's logs"
 
-let conf_level = Dtools.Conf.int ~p:(conf_log#plug "level") ~d:5 "Level"
+let conf_verbosity =
+  Dtools.Conf.string
+    ~p:(conf_log#plug "verbosity")
+    "Verbosity" ~d:"warning"
+    ~comments:
+      [
+        "Set SRT log level, one of: \"critical\", \"error\", ";
+        "\"warning\", \"notice\" or \"debug\"";
+      ]
+
+let conf_level = Dtools.Conf.int ~p:(conf_log#plug "level") ~d:4 "Level"
 let conf_poll = Dtools.Conf.void ~p:(conf_srt#plug "poll") "Poll configuration"
 
 let conf_timeout =
@@ -552,7 +562,20 @@ end
 let () =
   Srt.startup ();
   Lifecycle.before_start (fun () ->
-      if conf_log#get then Srt.Log.set_handler log_handler);
+      if conf_log#get then (
+        let level =
+          match conf_verbosity#get with
+            | "critical" -> `Critical
+            | "error" -> `Error
+            | "warning" -> `Warning
+            | "notice" -> `Notice
+            | "debug" -> `Debug
+            | _ ->
+                log#severe "Invalid value for \"srt.log.verbosity\"!";
+                `Error
+        in
+        Srt.Log.setloglevel level;
+        Srt.Log.set_handler log_handler));
   Lifecycle.after_scheduler_shutdown (fun () ->
       Srt.Poll.release Poll.t.Poll.p;
       Srt.cleanup ())
@@ -694,7 +717,11 @@ class virtual caller ~payload_size ~messageapi ~hostname ~port
     method private disconnect =
       self#mutexify
         (fun () ->
-          ignore (Option.map close_socket socket);
+          (match socket with
+            | None -> ()
+            | Some socket ->
+                close_socket socket;
+                !on_disconnect ());
           socket <- None;
           task_should_stop <- true;
           match connect_task with
@@ -868,15 +895,15 @@ class virtual input_base ~kind ~max ~log_overfull ~clock_safe ~on_connect
       let pos = Frame.position frame in
       try
         let socket = self#get_socket in
-        let decoder =
+        let decoder, buffer =
           match decoder_data with
             | None ->
+                let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
                 let decoder = self#create_decoder socket in
-                decoder_data <- Some decoder;
-                decoder
+                decoder_data <- Some (decoder, buffer);
+                (decoder, buffer)
             | Some d -> d
         in
-        let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
         while Generator.length generator < Lazy.force Frame.size do
           decoder.Decoder.decode buffer
         done;
