@@ -168,6 +168,10 @@ let throw print_error = function
       error_header 7 v.Term.Value.pos;
       Format.printf "Invalid value:@ %s@]@." msg;
       raise Error
+  | Lang_encoders.Error (term, msg) ->
+      error_header 8 term.Term.t.Type.pos;
+      Format.printf "%s.@]@." msg;
+      raise Error
   | Failure s ->
       print_error 9 (Printf.sprintf "Failure: %s" s);
       raise Error
