@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -150,10 +150,12 @@ let process_request s =
     | Request.Failed ->
         Printf.printf "Request resolution failed.\n";
         Request.destroy req;
+        flush_all ();
         exit 2
     | Request.Timeout ->
         Printf.printf "Request resolution timeout.\n";
         Request.destroy req;
+        flush_all ();
         exit 1
     | Request.Resolved ->
         let metadata = Request.get_all_metadata req in
@@ -337,7 +339,7 @@ let options =
             (fun () ->
               Printf.printf
                 "Liquidsoap %s\n\
-                 Copyright (c) 2003-2021 Savonet team\n\
+                 Copyright (c) 2003-2022 Savonet team\n\
                  Liquidsoap is open-source software, released under GNU \
                  General Public License.\n\
                  See <http://liquidsoap.info> for more information.\n"
@@ -460,6 +462,7 @@ let final_cleanup () =
   log#important "Cleaning downloaded files...";
   Request.clean ();
   log#important "Freeing memory...";
+  Gc.full_major ();
   Gc.full_major ()
 
 let sync_cleanup () =
@@ -501,6 +504,7 @@ let check_directories () =
         \  set(%S, \"<path>\")\n"
         kind dir
         (Dtools.Conf.string_of_path (List.hd routes));
+      flush_all ();
       exit 1)
   in
   if Dtools.Log.conf_file#get then (
@@ -530,6 +534,7 @@ let () =
         (* TODO if start fails (e.g. invalid password or mountpoint) it
          *   raises an exception and dtools catches it so we don't get
          *   a backtrace (by default at least). *)
+        Server.start ();
         Clock.start ();
         Tutils.start ();
         Tutils.main ()
@@ -560,6 +565,7 @@ let () =
         else (
           final_cleanup ();
           Printf.printf "No output defined, nothing to do.\n";
+          flush_all ();
           exit 1))
 
 (* Here we go! *)
