@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ let finalise_child_clock child_clock source =
 (** [rms_width] and [minimum_length] are all in samples.
   * [cross_length] is in ticks (like #remaining estimations).
   * We are assuming a fixed audio kind -- at least for now. *)
-class cross ~kind val_source ~cross_length ~override_duration ~rms_width
+class cross ~kind val_source ~duration_getter ~override_duration ~rms_width
   ~minimum_length ~conservative ~active transition =
   let s = Lang.to_source val_source in
   object (self)
@@ -47,7 +47,23 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
      * sources but we do not have a static way of knowing it at the moment.
      * Going with the same choice as above for now. *)
     method self_sync = s#self_sync
-    val mutable cross_length = cross_length ()
+    val mutable cross_length = 0
+    val mutable duration_getter = duration_getter
+
+    method set_cross_length =
+      let new_cross_length = duration_getter () in
+      let main_new_cross_length = Frame.main_of_seconds new_cross_length in
+
+      if main_new_cross_length <> cross_length then
+        if new_cross_length < 0. then
+          self#log#important
+            "Cannot set crossfade duration to negative value %f!"
+            new_cross_length
+        else (
+          self#log#info "Setting crossfade duration to %.2fs" new_cross_length;
+          cross_length <- main_new_cross_length)
+
+    initializer self#set_cross_length
 
     (* We need to store the end of a track, and compute the power of the signal
      * before the end of track. For doing so we need to remember a sliding window
@@ -91,6 +107,9 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
 
     (* Give a default value for the transition source. *)
     val mutable transition_source = None
+
+    (* This is used to track the state of the transition source and switch back
+       to the current source w/o introducing artificial track marks. *)
     val mutable pending_after = Generator.create ()
 
     method private prepare_transition_source s =
@@ -137,7 +156,7 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
 
     method private child_tick =
       child_support#child_tick;
-      Frame.advance buf_frame;
+      Frame.clear buf_frame;
       last_child_tick <- (Clock.get self#clock)#get_tick
 
     method before_output =
@@ -172,23 +191,20 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
               | None -> ()
               | Some v -> (
                   try
+                    self#log#info
+                      "Overriding crossfade duration from metadata %s"
+                      override_duration;
                     let l = float_of_string v in
-                    self#log#info "Setting crossfade duration to %.2fs" l;
-                    cross_length <- Frame.main_of_seconds l
+                    duration_getter <- (fun () -> l)
                   with _ -> ())))
-        (Frame.get_all_metadata frame)
+        (Frame.get_all_metadata frame);
+      self#set_cross_length
 
     method private get_frame frame =
       match status with
         | `Idle ->
-            let rem = if conservative then 0 else source#remaining in
-            if rem < 0 || rem > cross_length then (
-              let p = Frame.position frame in
-              source#get frame;
-              self#save_last_metadata `Before frame;
-              self#update_cross_length frame p;
-              needs_tick <- true)
-            else (
+            let rem = source#remaining in
+            if conservative || (0 <= rem && rem <= cross_length) then (
               self#log#info "Buffering end of track...";
               status <- `Before;
               Frame.set_breaks buf_frame [Frame.position frame];
@@ -197,6 +213,12 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
               if status <> `Limit then
                 self#log#info "More buffering will be needed.";
               self#get_frame frame)
+            else (
+              let p = Frame.position frame in
+              source#get frame;
+              self#save_last_metadata `Before frame;
+              self#update_cross_length frame p;
+              needs_tick <- true)
         | `Before ->
             (* We started buffering but the track didn't end.
              * Play the beginning of the buffer while filling it more. *)
@@ -262,19 +284,12 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
 
       (* Analyze them *)
       let pcm = AFrame.pcm buf_frame in
-      for i = start to stop - 1 do
-        let squares =
-          Array.fold_left
-            (fun squares track ->
-              let x = track.{i} in
-              squares +. (x *. x))
-            0. pcm
-        in
-        rms_before <- rms_before -. mem_before.(mem_i) +. squares;
-        mem_before.(mem_i) <- squares;
-        mem_i <- (mem_i + 1) mod rms_width;
-        rmsi_before <- min rms_width (rmsi_before + 1)
-      done;
+      let len = stop - start in
+      let squares = Audio.squares pcm start len in
+      rms_before <- rms_before -. mem_before.(mem_i) +. squares;
+      mem_before.(mem_i) <- squares;
+      mem_i <- (mem_i + len) mod rms_width;
+      rmsi_before <- min rms_width (rmsi_before + len);
 
       (* Should we buffer more or are we done ? *)
       if AFrame.is_partial buf_frame then (
@@ -299,17 +314,10 @@ class cross ~kind val_source ~cross_length ~override_duration ~rms_width
         let after_len = Generator.length gen_after in
         if after_len <= rms_width then (
           let pcm = AFrame.pcm buf_frame in
-          for i = start to stop - 1 do
-            let squares =
-              Array.fold_left
-                (fun squares track ->
-                  let x = track.{i} in
-                  squares +. (x *. x))
-                0. pcm
-            in
-            rms_after <- rms_after +. squares;
-            rmsi_after <- rmsi_after + 1
-          done);
+          let len = stop - start in
+          let squares = Audio.squares pcm start len in
+          rms_after <- rms_after +. squares;
+          rmsi_after <- rmsi_after + len);
         self#save_last_metadata `After buf_frame;
         self#update_cross_length buf_frame start;
         if AFrame.is_partial buf_frame && not source#is_ready then
@@ -481,11 +489,10 @@ let () =
        depending on the relative power of the signal before and after the end \
        of track."
     (fun p ->
-      let duration = Lang.to_float_getter (List.assoc "duration" p) in
+      let duration_getter = Lang.to_float_getter (List.assoc "duration" p) in
       let override_duration =
         Lang.to_string (List.assoc "override_duration" p)
       in
-      let cross_length () = Frame.main_of_seconds (duration ()) in
       let minimum = Lang.to_float (List.assoc "minimum" p) in
       let minimum_length = Frame.audio_of_seconds minimum in
       let rms_width = Lang.to_float (List.assoc "width" p) in
@@ -497,7 +504,7 @@ let () =
       let kind = Source.Kind.of_kind kind in
       let c =
         new cross
-          ~kind source transition ~conservative ~active ~cross_length ~rms_width
-          ~minimum_length ~override_duration
+          ~kind source transition ~conservative ~active ~duration_getter
+          ~rms_width ~minimum_length ~override_duration
       in
       (c :> source))
