@@ -1,7 +1,7 @@
 (*****************************************************************************
 
   Liquidsoap, a programmable audio stream generator.
-  Copyright 2003-2021 Savonet team
+  Copyright 2003-2022 Savonet team
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -153,8 +153,7 @@ module Make (Harbor : T) = struct
           { Decoder.read; tell = None; length = None; lseek = None }
         in
         try
-          let decoder = create_decoder input in
-          let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
+          let decoder, buffer = create_decoder input in
           while true do
             Tutils.mutexify relay_m
               (fun () ->
@@ -200,8 +199,12 @@ module Make (Harbor : T) = struct
           with Not_found -> mime
         in
         match Decoder.get_stream_decoder ~ctype:self#ctype mime with
-          | Some d ->
-              create_decoder <- d;
+          | Some decoder ->
+              let decoder args =
+                let buffer = Decoder.mk_buffer ~ctype:self#ctype generator in
+                (decoder args, buffer)
+              in
+              create_decoder <- decoder;
               mime_type <- Some mime
           | None -> raise Harbor.Unknown_codec
 
@@ -298,11 +301,16 @@ module Make (Harbor : T) = struct
       [
         ( "buffer",
           Lang.float_t,
-          Some (Lang.float 2.),
-          Some "Duration of the pre-buffered data." );
+          Some (Lang.float 12.),
+          Some
+            "Duration of the pre-buffered data. Default value is set to make \
+             it possible to use `crossfade` transitions with `input.harbor`. \
+             You might be able to reduce it but, in this case, make sure to \
+             not use the operator with `crossfade` or make sure that it has \
+             enough buffered data for it." );
         ( "max",
           Lang.float_t,
-          Some (Lang.float 10.),
+          Some (Lang.float 20.),
           Some "Maximum duration of the buffered data." );
         ( "timeout",
           Lang.float_t,
